//----------------------------------------------------------------------------
//  glBSP Win32 About Dialog
//----------------------------------------------------------------------------
// 
//  Copyright (c) 2000  Andy Baker
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//----------------------------------------------------------------------------
//
#include "sys_inc.h"
#include "gui.h"

// About Box General
#define ABOUT_TITLE           "About glBSP"
#define ABOUT_WINDOW_WIDTH    365
#define ABOUT_WINDOW_HEIGHT   105

// OK Button
#define ABOUT_OKBUTTON_X      140
#define ABOUT_OKBUTTON_Y      50
#define ABOUT_OKBUTTON_WIDTH  80
#define ABOUT_OKBUTTON_HEIGHT 25
#define ABOUT_OKBUTTON_TEXT   "OK"

// About Text
#define ABOUT_TEXT_X      10
#define ABOUT_TEXT_Y      10
#define ABOUT_TEXT_WIDTH  340
#define ABOUT_TEXT_HEIGHT 40

static const char *ABOUT_TEXT =
  "glBSP Node Builder " GLBSP_VER " - (C) 2000 Andrew Apted.\n"
  "Based on BSP 2.3 (c) 1998 Colin Reed, Lee Killough\n"
  "\n"
  "Credits should go to :-\n"
  "  Andy Baker & Marc Pullen   for their invaluable help\n"
  "  Colin Reed & Lee Killough  for creating the original BSP\n"
  "  Matt Fell                  for the Doom Specs\n"
  "  Raphael Quinet             for DEU and the original idea\n"
  "  ... and everyone who helped with the original BSP.\n"
  "\n"
  "This program is free software, under the terms of the GNU General\n"
  "Public License, and comes with ABSOLUTELY NO WARRANTY.  See the\n"
  "accompanying documentation for more details.";


// About Box Control
static boolean_g aboutboxinit = false;
static HWND aboutboxhwnd;
static HINSTANCE aboutboxinst;

// Handles
static int okbutton = -1;
static int abouttext = -1;

// ============================ ABOUT WINDOW ============================

// Prototypes
boolean_g AboutProcCommandMessage(HWND window, HWND ctrl, DWORD command);

boolean_g AboutInitControls(void);
boolean_g AboutInitOKButton(HWND wnd, HINSTANCE inst);
boolean_g AboutInitText(HWND wnd, HINSTANCE inst);

//
// AboutMsgHandler
//
// Message handling Loop
//
long AboutMsgHandler(HWND window, UINT message, WPARAM wParam, LPARAM lParam)
{
  switch (message)
  {
    case WM_COMMAND:
      if (AboutProcCommandMessage(window, (HWND)(lParam), LOWORD(wParam)))
        return 0L;
      break;

    case WM_CLOSE:
      if (okbutton>=0)
      {
        I_GUIDestroyButton(okbutton);
        okbutton = -1;
      }

      if (abouttext>=0)
      {
        I_GUIDestroyText(abouttext);
        abouttext = -1;
      }

      I_GUIRemoveMessageHandler(aboutboxhwnd);
      aboutboxinit = false;
      aboutboxhwnd = NULL;
      break;
  }

  return DefWindowProc(window, message, wParam, lParam);
}

//
// AboutWindowStartup
//
// Init Main Window
//
boolean_g AboutWindowStartup(HWND parentwnd, HINSTANCE inst)
{
  WNDCLASS wc;
  msgfunc_t messagefunc;
  char *classname;
  RECT winrect;
  int x, y;

  if (aboutboxinit)
  {
    SetFocus(aboutboxhwnd);
    return true;
  }

  if (!I_GUIRegisterClass(inst, CLTYPE_POPUP))
    return false;

  classname = I_GUIReturnClassName(CLTYPE_POPUP);
  if (!classname)
    return false;

  aboutboxinst = inst;

  GetWindowRect(parentwnd, &winrect);
  x = winrect.left + 100;
  y = winrect.top  + 100;

  // Create a window
  aboutboxhwnd = CreateWindowEx(WS_EX_WINDOWEDGE,
                                classname,
                                ABOUT_TITLE,
                                WS_OVERLAPPED|WS_CAPTION,
                                x,
                                y,
                                ABOUT_WINDOW_WIDTH,
                                ABOUT_WINDOW_HEIGHT,
                                parentwnd,
                                NULL,
                                inst,
                                NULL);

  if (!aboutboxhwnd)
    return false;

  // Add Message Handler
  messagefunc.winhandle = aboutboxhwnd;
  messagefunc.handler = AboutMsgHandler;
  if (!I_GUIAddMessageHandler(&messagefunc))
  {
    DestroyWindow(aboutboxhwnd);
    return false;
  }

  if (!AboutInitControls())
    return false;

  ShowWindow(aboutboxhwnd, SW_SHOWNORMAL);
  UpdateWindow(aboutboxhwnd);
  SetFocus(aboutboxhwnd);

  aboutboxinit = true;
  return true;
}

//
// AboutProcCommandMessage
//
// Handle WM_COMMAND Messages
//
boolean_g AboutProcCommandMessage(HWND window, HWND ctrl, DWORD command)
{
  if (I_GUIButtonMsghandler(ctrl, command))
    return true;

  return false;
}

//
// AboutInitControls
//
boolean_g AboutInitControls(void)
{
  if (!AboutInitOKButton(aboutboxhwnd, aboutboxinst))
    return false;

  if (!AboutInitText(aboutboxhwnd, aboutboxinst))
    return false;

  return true;
}

//
// AboutWindowShutdown
//
void AboutWindowShutdown(void)
{
  SendMessage(aboutboxhwnd, WM_CLOSE, 0, 0);
  return;
}

// =========================== "OK" BUTTON =============================

//
// AboutOKButtonHandler
//
void AboutOKButtonHandler(HWND window, DWORD message)
{
  if (message == BN_CLICKED)
  {
    SendMessage(aboutboxhwnd, WM_CLOSE, 0, 0);
    return;
  }

  return;
}

//
// AboutInitOKButton
//
boolean_g AboutInitOKButton(HWND wnd, HINSTANCE inst)
{
  guibutton_t button;

  // Already inited?
  if (okbutton>=0)
    return false;

  memset(&button, 0, sizeof(guibutton_t));

  button.parentwindow     = wnd;
  button.parentinst       = inst;
  button.x                = ABOUT_OKBUTTON_X;
  button.y                = ABOUT_OKBUTTON_Y;
  button.width            = ABOUT_OKBUTTON_WIDTH;
  button.height           = ABOUT_OKBUTTON_HEIGHT;
  button.text             = ABOUT_OKBUTTON_TEXT;
  button.action           = AboutOKButtonHandler;

  okbutton = I_GUICreateButton(&button);

  if (okbutton < 0)
    return false;

  return true;
}

// =========================== ABOUT TEXT =============================

//
// AboutInitText
//
boolean_g AboutInitText(HWND wnd, HINSTANCE inst)
{
  guitext_t gtext;

  // Already inited?
  if (abouttext>=0)
    return false;

  memset(&gtext, 0, sizeof(guitext_t));

  gtext.parentwindow     = wnd;
  gtext.parentinst       = inst;
  gtext.x                = ABOUT_TEXT_X;
  gtext.y                = ABOUT_TEXT_Y;
  gtext.width            = ABOUT_TEXT_WIDTH;
  gtext.height           = ABOUT_TEXT_HEIGHT;
  gtext.text             = ABOUT_TEXT;

  abouttext = I_GUICreateText(&gtext);

  if (abouttext < 0)
    return false;

  return true;
}


// =====================================================================

