//----------------------------------------------------------------------------
//  glBSP Win32 Node Build Progression Box
//----------------------------------------------------------------------------
// 
//  Copyright (c) 2000  Andy Baker
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//----------------------------------------------------------------------------
//
#include "sys_inc.h"
#include "gui.h"

// Build Progress Box General
#define BUILD_TITLE               "BUILD PROGRESS DIALOG BOX"
#define BUILD_WINDOW_WIDTH        365
#define BUILD_WINDOW_HEIGHT       150

// CANCEL Button
#define BUILD_CANCELBUTTON_X      140
#define BUILD_CANCELBUTTON_Y      95
#define BUILD_CANCELBUTTON_WIDTH  80
#define BUILD_CANCELBUTTON_HEIGHT 25
#define BUILD_CANCELBUTTON_TEXT   "Cancel"

// Build Info Text
#define BUILD_TEXT_X              5
#define BUILD_TEXT_Y              5
#define BUILD_TEXT_WIDTH          350
#define BUILD_TEXT_HEIGHT         20
#define BUILD_TEXT                "BUILD BAR TEXT (ID: 1)" // Updated by user function

// Build Progress Bar
#define BUILD_PBAR_X              5
#define BUILD_PBAR_Y              25
#define BUILD_PBAR_WIDTH          350
#define BUILD_PBAR_HEIGHT         20
#define BUILD_PBAR_MAXRANGE       100
#define BUILD_PBAR_STEP           1

// Build Info Text 2
#define BUILD_TEXT2_X             5
#define BUILD_TEXT2_Y             50
#define BUILD_TEXT2_WIDTH         350
#define BUILD_TEXT2_HEIGHT        20
#define BUILD_TEXT2               "BUILD BAR TEXT (ID: 2)" // Updated by user function

// Build Progress Bar 2
#define BUILD_PBAR2_X             5
#define BUILD_PBAR2_Y             70
#define BUILD_PBAR2_WIDTH         350
#define BUILD_PBAR2_HEIGHT        20
#define BUILD_PBAR2_MAXRANGE      100
#define BUILD_PBAR2_STEP          1

// FILEPROC Box Control
static boolean_g buildinit = false;
static HWND buildhwnd;
static HINSTANCE buildinst;

// Handles
static int cancelbutton = -1;
static int buildtext = -1;
static int buildpbar = -1;
static int buildtext2 = -1;
static int buildpbar2 = -1;

// ======================= BUILD PROGRESS WINDOW =======================

// Prototypes
boolean_g BuildProcCommandMessage(HWND window, HWND ctrl, DWORD command);

boolean_g BuildInitControls(void);
boolean_g BuildInitCancelButton(HWND wnd, HINSTANCE inst);
boolean_g BuildInitText(HWND wnd, HINSTANCE inst);
boolean_g BuildInitProgressBar(HWND wnd, HINSTANCE inst);
boolean_g BuildInitText2(HWND wnd, HINSTANCE inst);
boolean_g BuildInitProgressBar2(HWND wnd, HINSTANCE inst);

void BuildKillControls(void);

//
// BuildMsgHandler
//
// Message handling Loop
//
long BuildMsgHandler(HWND window, UINT message, WPARAM wParam, LPARAM lParam)
{
  switch (message)
  {
    case WM_COMMAND:
      if (BuildProcCommandMessage(window, (HWND)(lParam), LOWORD(wParam)))
        return 0L;
      break;

    case WM_CLOSE:
      BuildKillControls();
      I_GUIRemoveMessageHandler(buildhwnd);
      buildinit = false;
      buildhwnd = NULL;
      break;
  }

  return DefWindowProc(window, message, wParam, lParam);
}

//
// BuildWindowStartup
//
// Init Main Window
//
boolean_g BuildWindowStartup(HWND parentwnd, HINSTANCE inst)
{
  WNDCLASS wc;
  msgfunc_t messagefunc;
  char *classname;
  RECT winrect;
  int x, y;

  if (buildinit)
  {
    SetFocus(buildhwnd);
    return true;
  }

  if (!I_GUIRegisterClass(inst, CLTYPE_POPUP))
    return false;

  classname = I_GUIReturnClassName(CLTYPE_POPUP);
  if (!classname)
    return false;

  buildinst = inst;

  GetWindowRect(parentwnd, &winrect);
  x = winrect.left + 100;
  y = winrect.top  + 100;

  // Create a window
  buildhwnd = CreateWindowEx(WS_EX_WINDOWEDGE,
                                classname,
                                BUILD_TITLE,
                                WS_OVERLAPPED|WS_CAPTION,
                                x,
                                y,
                                BUILD_WINDOW_WIDTH,
                                BUILD_WINDOW_HEIGHT,
                                parentwnd,
                                NULL,
                                inst,
                                NULL);

  if (!buildhwnd)
    return false;

  // Add Message Handler
  messagefunc.winhandle = buildhwnd;
  messagefunc.handler = BuildMsgHandler;
  if (!I_GUIAddMessageHandler(&messagefunc))
  {
    DestroyWindow(buildhwnd);
    return false;
  }

  if (!BuildInitControls())
    return false;

  ShowWindow(buildhwnd, SW_SHOWNORMAL);
  UpdateWindow(buildhwnd);
  SetFocus(buildhwnd);

  buildinit = true;
  return true;
}

//
// BuildProcCommandMessage
//
// Handle WM_COMMAND Messages
//
boolean_g BuildProcCommandMessage(HWND window, HWND ctrl, DWORD command)
{
  if (I_GUIButtonMsghandler(ctrl, command))
    return true;

  return false;
}

//
// BuildInitControls
//
boolean_g BuildInitControls(void)
{
  if (!BuildInitCancelButton(buildhwnd, buildinst))
    return false;

  if (!BuildInitText(buildhwnd, buildinst))
    return false;

  if (!BuildInitProgressBar(buildhwnd, buildinst))
    return false;

  if (!BuildInitText2(buildhwnd, buildinst))
    return false;

  if (!BuildInitProgressBar2(buildhwnd, buildinst))
    return false;

  return true;
}

//
// BuildKillControls
//
void BuildKillControls(void)
{
  if (cancelbutton>=0)
  {
    I_GUIDestroyButton(cancelbutton);
    cancelbutton = -1;
  }

  if (buildtext>=0)
  {
    I_GUIDestroyText(buildtext);
    buildtext = -1;
  }

  if (buildpbar>=0)
  {
    I_GUIDestroyBar(buildpbar);
    buildpbar = -1;
  }

  if (buildtext2>=0)
  {
    I_GUIDestroyText(buildtext2);
    buildtext2 = -1;
  }

  if (buildpbar2>=0)
  {
    I_GUIDestroyBar(buildpbar2);
    buildpbar2 = -1;
  }

  return;
}

//
// BuildWindowSetTitle
//
boolean_g BuildWindowSetTitle(const char *str)
{
  if (!str)
    return false;

  SetWindowText(buildhwnd, str);
  return true;
}

//
// BuildWindowShutdown
//
void BuildWindowShutdown(void)
{
  SendMessage(buildhwnd, WM_CLOSE, 0, 0);
  return;
}

// ========================== "CANCEL" BUTTON =============================

//
// BuildCancelButtonHandler
//
void BuildCancelButtonHandler(HWND window, DWORD message)
{
  if (message == BN_CLICKED)
  {
    SendMessage(buildhwnd, WM_CLOSE, 0, 0);
    return;
  }

  return;
}

//
// BuildInitCancelButton
//
boolean_g BuildInitCancelButton(HWND wnd, HINSTANCE inst)
{
  guibutton_t button;

  // Already inited?
  if (cancelbutton>=0)
    return false;

  memset(&button, 0, sizeof(guibutton_t));

  button.parentwindow     = wnd;
  button.parentinst       = inst;
  button.x                = BUILD_CANCELBUTTON_X;
  button.y                = BUILD_CANCELBUTTON_Y;
  button.width            = BUILD_CANCELBUTTON_WIDTH;
  button.height           = BUILD_CANCELBUTTON_HEIGHT;
  button.text             = BUILD_CANCELBUTTON_TEXT;
  button.action           = BuildCancelButtonHandler;

  cancelbutton = I_GUICreateButton(&button);

  if (cancelbutton < 0)
    return false;

  return true;
}

// ============================= BUILD TEXT ===============================

//
// BuildInitText
//
boolean_g BuildInitText(HWND wnd, HINSTANCE inst)
{
  guitext_t gtext;

  // Already inited?
  if (buildtext>=0)
    return false;

  memset(&gtext, 0, sizeof(guitext_t));

  gtext.parentwindow     = wnd;
  gtext.parentinst       = inst;
  gtext.x                = BUILD_TEXT_X;
  gtext.y                = BUILD_TEXT_Y;
  gtext.width            = BUILD_TEXT_WIDTH;
  gtext.height           = BUILD_TEXT_HEIGHT;
  gtext.text             = BUILD_TEXT;

  buildtext = I_GUICreateText(&gtext);

  if (buildtext < 0)
    return false;

  return true;
}

// ============================ PROGRESS BAR ==============================

//
// BuildInitProgressBar
//
boolean_g BuildInitProgressBar(HWND wnd, HINSTANCE inst)
{
  progressbar_t bar;

  // Already inited?
  if (buildpbar>=0)
    return false;

  memset(&bar, 0, sizeof(progressbar_t));

  bar.parentwindow     = wnd;
  bar.parentinst       = inst;
  bar.x                = BUILD_PBAR_X;
  bar.y                = BUILD_PBAR_Y;
  bar.width            = BUILD_PBAR_WIDTH;
  bar.height           = BUILD_PBAR_HEIGHT;
  bar.maxrange         = BUILD_PBAR_MAXRANGE;
  bar.step             = BUILD_PBAR_STEP;
  bar.barcolour.blue   = 224;

  buildpbar = I_GUICreateBar(&bar);

  if (buildpbar<0)
    return false;                                     

  return true;
}

// ============================= BUILD TEXT 2 ===============================

//
// BuildInitText2
//
boolean_g BuildInitText2(HWND wnd, HINSTANCE inst)
{
  guitext_t gtext;

  // Already inited?
  if (buildtext2>=0)
    return false;

  memset(&gtext, 0, sizeof(guitext_t));

  gtext.parentwindow     = wnd;
  gtext.parentinst       = inst;
  gtext.x                = BUILD_TEXT2_X;
  gtext.y                = BUILD_TEXT2_Y;
  gtext.width            = BUILD_TEXT2_WIDTH;
  gtext.height           = BUILD_TEXT2_HEIGHT;
  gtext.text             = BUILD_TEXT2;

  buildtext2 = I_GUICreateText(&gtext);

  if (buildtext2 < 0)
    return false;

  return true;
}

// ============================ PROGRESS BAR 2 ==============================

//
// BuildInitProgressBar2
//
boolean_g BuildInitProgressBar2(HWND wnd, HINSTANCE inst)
{
  progressbar_t bar;

  // Already inited?
  if (buildpbar2>=0)
    return false;

  memset(&bar, 0, sizeof(progressbar_t));

  bar.parentwindow     = wnd;
  bar.parentinst       = inst;
  bar.x                = BUILD_PBAR2_X;
  bar.y                = BUILD_PBAR2_Y;
  bar.width            = BUILD_PBAR2_WIDTH;
  bar.height           = BUILD_PBAR2_HEIGHT;
  bar.maxrange         = BUILD_PBAR2_MAXRANGE;
  bar.step             = BUILD_PBAR2_STEP;
  bar.barcolour.blue   = 224;

  buildpbar2 = I_GUICreateBar(&bar);

  if (buildpbar2<0)
    return false;                                     

  return true;
}

// ========================= BUILD TEXT - SHARED ============================

//
// BuildUpdateBarText
//
boolean_g BuildUpdateBarText(int num, const char *str)
{
  if (num==1 && buildtext>=0)
    return I_GUISetText(buildtext, str);

  if (num==2 && buildtext2>=0)
    return I_GUISetText(buildtext2, str);

  return false;
}

// ======================== PROGRESS BAR - SHARED ==========================

//
// BuildUpdateBar
//
boolean_g BuildUpdateBar(int num, int val)
{
  if (num==1 && buildpbar>=0)
    return I_GUIUpdateBar(buildpbar, val);

  if (num==2 && buildpbar2>=0)
    return I_GUIUpdateBar(buildpbar2, val);

  return false;
}

//
// BuildUpdateBarLimit
//
boolean_g BuildUpdateBarLimit(int num, int val)
{
  if (num==1 && buildpbar>=0)
    return I_GUIUpdateBarRange(buildpbar, 0, val);

  if (num==2 && buildpbar2>=0)
    return I_GUIUpdateBarRange(buildpbar2, 0, val);

  return false;
}

// =====================================================================

//
// I_InitBuildProcessBox
//
boolean_g I_InitBuildProcessDisplay(void)
{
  return false;
}

//
// I_KillBuildProcessBox
//
void I_KillBuildProcessDisplay(void)
{
  return;
}

