//----------------------------------------------------------------------------
//  glBSP Win32 Load/Save Progress Box
//----------------------------------------------------------------------------
// 
//  Copyright (c) 2000  Andy Baker
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//----------------------------------------------------------------------------
//
#include "sys_inc.h"
#include "gui.h"

// File Processing Box General
#define FILEPROC_TITLE               "FILE PROGRESS TITLE"
#define FILEPROC_WINDOW_WIDTH        365
#define FILEPROC_WINDOW_HEIGHT       105

// CANCEL Button
#define FILEPROC_CANCELBUTTON_X      140
#define FILEPROC_CANCELBUTTON_Y      50
#define FILEPROC_CANCELBUTTON_WIDTH  80
#define FILEPROC_CANCELBUTTON_HEIGHT 25
#define FILEPROC_CANCELBUTTON_TEXT   "Cancel"

// File Progress Bar
#define FILEPROC_PBAR_X              5
#define FILEPROC_PBAR_Y              25
#define FILEPROC_PBAR_WIDTH          350
#define FILEPROC_PBAR_HEIGHT         20
#define FILEPROC_PBAR_MAXRANGE       100
#define FILEPROC_PBAR_STEP           1

// File Info Text
#define FILEPROC_TEXT_X              5
#define FILEPROC_TEXT_Y              5
#define FILEPROC_TEXT_WIDTH          350
#define FILEPROC_TEXT_HEIGHT         20
#define FILEPROC_TEXT                "BAR TEXT" // Updated by user function

// FILEPROC Box Control
static boolean_g fileprocinit = false;
static HWND fileprochwnd;
static HINSTANCE fileprocinst;

// Handles
static int cancelbutton = -1;
static int fileproctext = -1;
static int fileprocpbar = -1;

// ======================= FILE PROCESS WINDOW =======================

// Prototypes
boolean_g FileProcCommandMessage(HWND window, HWND ctrl, DWORD command);

boolean_g FileProcInitControls(void);
boolean_g FileProcInitCancelButton(HWND wnd, HINSTANCE inst);
boolean_g FileProcInitProgressBar(HWND wnd, HINSTANCE inst);
boolean_g FileProcInitBarText(HWND wnd, HINSTANCE inst);
void FileProcKillControls(void);

//
// FileProcMsgHandler
//
// Message handling Loop
//
long FileProcMsgHandler(HWND window, UINT message, WPARAM wParam, LPARAM lParam)
{
  switch (message)
  {
    case WM_COMMAND:
      if (FileProcCommandMessage(window, (HWND)(lParam), LOWORD(wParam)))
        return 0L;
      break;

    case WM_CLOSE:
      FileProcKillControls();
      I_GUIRemoveMessageHandler(fileprochwnd);
      fileprocinit = false;
      fileprochwnd = NULL;
      break;
  }

  return DefWindowProc(window, message, wParam, lParam);
}

//
// FileProcWindowStartup
//
// Init Main Window
//
boolean_g FileProcWindowStartup(HWND parentwnd, HINSTANCE inst)
{
  WNDCLASS wc;
  msgfunc_t messagefunc;
  char *classname;
  RECT winrect;
  int x, y;

  if (fileprocinit)
  {
    SetFocus(fileprochwnd);
//    return true;
    return false;
  }

  if (!I_GUIRegisterClass(inst, CLTYPE_POPUP))
    return false;

  classname = I_GUIReturnClassName(CLTYPE_POPUP);
  if (!classname)
    return false;

  fileprocinst = inst;

  GetWindowRect(parentwnd, &winrect);
  x = winrect.left + 100;
  y = winrect.top  + 100;

  // Create a window
  fileprochwnd = CreateWindowEx(WS_EX_WINDOWEDGE,
                                classname,
                                FILEPROC_TITLE,
                                WS_OVERLAPPED|WS_CAPTION,
                                x,
                                y,
                                FILEPROC_WINDOW_WIDTH,
                                FILEPROC_WINDOW_HEIGHT,
                                parentwnd,
                                NULL,
                                inst,
                                NULL);

  if (!fileprochwnd)
    return false;

  // Add Message Handler
  messagefunc.winhandle = fileprochwnd;
  messagefunc.handler = FileProcMsgHandler;
  if (!I_GUIAddMessageHandler(&messagefunc))
  {
    DestroyWindow(fileprochwnd);
    return false;
  }

  if (!FileProcInitControls())
    return false;

  ShowWindow(fileprochwnd, SW_SHOWNORMAL);
  UpdateWindow(fileprochwnd);
  SetFocus(fileprochwnd);

  fileprocinit = true;
  return true;
}

//
// FileProcWindowSetTitle
//
boolean_g FileProcWindowSetTitle(const char *str)
{
  if (!str)
    return false;

  SetWindowText(fileprochwnd, str);
  return true;
}

//
// FileProcCommandMessage
//
// Handle WM_COMMAND Messages
//
boolean_g FileProcCommandMessage(HWND window, HWND ctrl, DWORD command)
{
  if (I_GUIButtonMsghandler(ctrl, command))
    return true;

  return false;
}

//
// FileProcInitControls
//
boolean_g FileProcInitControls(void)
{
  if (!FileProcInitCancelButton(fileprochwnd, fileprocinst))
    return false;

  if (!FileProcInitProgressBar(fileprochwnd, fileprocinst))
    return false;

  if (!FileProcInitBarText(fileprochwnd, fileprocinst))
    return false;

  return true;
}

//
// FileProcKillControls
//
void FileProcKillControls(void)
{
  if (cancelbutton>=0)
  {
    I_GUIDestroyButton(cancelbutton);
    cancelbutton = -1;
  }

  if (fileprocpbar>=0)
  {
    I_GUIDestroyBar(fileprocpbar);
    fileprocpbar = -1;
  }

  if (fileproctext>=0)
  {
    I_GUIDestroyText(fileproctext);
    fileproctext = -1;
  }

  return;
}

//
// FileProcWindowShutdown
//
void FileProcWindowShutdown(void)
{
  SendMessage(fileprochwnd, WM_CLOSE, 0, 0);
  return;
}

// ========================== "CANCEL" BUTTON =============================

//
// FileProcCancelButtonHandler
//
void FileProcCancelButtonHandler(HWND window, DWORD message)
{
  if (message == BN_CLICKED)
  {
    SendMessage(fileprochwnd, WM_CLOSE, 0, 0);
    return;
  }

  return;
}

//
// FileProcInitCancelButton
//
boolean_g FileProcInitCancelButton(HWND wnd, HINSTANCE inst)
{
  guibutton_t button;

  // Already inited?
  if (cancelbutton>=0)
    return false;

  memset(&button, 0, sizeof(guibutton_t));

  button.parentwindow     = wnd;
  button.parentinst       = inst;
  button.x                = FILEPROC_CANCELBUTTON_X;
  button.y                = FILEPROC_CANCELBUTTON_Y;
  button.width            = FILEPROC_CANCELBUTTON_WIDTH;
  button.height           = FILEPROC_CANCELBUTTON_HEIGHT;
  button.text             = FILEPROC_CANCELBUTTON_TEXT;
  button.action           = FileProcCancelButtonHandler;

  cancelbutton = I_GUICreateButton(&button);

  if (cancelbutton < 0)
    return false;

  return true;
}

// ============================ PROGRESS BAR ==============================

//
// FileProcInitProgressBar
//
boolean_g FileProcInitProgressBar(HWND wnd, HINSTANCE inst)
{
  progressbar_t bar;

  // Already inited?
  if (fileprocpbar>=0)
    return false;

  memset(&bar, 0, sizeof(progressbar_t));

  bar.parentwindow     = wnd;
  bar.parentinst       = inst;
  bar.x                = FILEPROC_PBAR_X;
  bar.y                = FILEPROC_PBAR_Y;
  bar.width            = FILEPROC_PBAR_WIDTH;
  bar.height           = FILEPROC_PBAR_HEIGHT;
  bar.maxrange         = FILEPROC_PBAR_MAXRANGE;
  bar.step             = FILEPROC_PBAR_STEP;
  bar.barcolour.blue   = 224;

  fileprocpbar = I_GUICreateBar(&bar);

  if (fileprocpbar<0)
    return false;                                     

  return true;
}

//
// FileProcUpdateBar
//
boolean_g FileProcUpdateBar(int val)
{
  if (fileprocpbar>=0)
    return I_GUIUpdateBar(fileprocpbar, val);
      
  return false;
}

//
// FileProcUpdateBarLimit
//
boolean_g FileProcUpdateBarLimit(int val)
{
  if (fileprocpbar>=0)
    return I_GUIUpdateBarRange(fileprocpbar, 0, val);
    
  return false;
}


// =========================== FILE INFO TEXT =============================

//
// FileProcInitBarText
//
boolean_g FileProcInitBarText(HWND wnd, HINSTANCE inst)
{
  guitext_t gtext;

  // Already inited?
  if (fileproctext>=0)
    return false;

  memset(&gtext, 0, sizeof(guitext_t));

  gtext.parentwindow     = wnd;
  gtext.parentinst       = inst;
  gtext.x                = FILEPROC_TEXT_X;
  gtext.y                = FILEPROC_TEXT_Y;
  gtext.width            = FILEPROC_TEXT_WIDTH;
  gtext.height           = FILEPROC_TEXT_HEIGHT;
  gtext.text             = FILEPROC_TEXT;

  fileproctext = I_GUICreateText(&gtext);

  if (fileproctext < 0)
    return false;

  return true;
}

//
// FileProcUpdateText
//
boolean_g FileProcUpdateBarText(const char *str)
{
  if (fileproctext>=0)
    return I_GUISetText(fileproctext, str);

  return false;
}

// =====================================================================

//
// I_InitFileProcessDisplay
//
boolean_g I_InitFileProcessDisplay(void)
{
  return false;
}

//
// I_KillFileProcessDisplay
//
void I_KillFileProcessDisplay(void)
{
  return;
}

