//----------------------------------------------------------------------------
//  glBSP Win32 Main Dialog Box
//----------------------------------------------------------------------------
// 
//  Copyright (c) 2000  Andy Baker
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//----------------------------------------------------------------------------
//
#include "sys_inc.h"
#include "gui.h"

#define TITLE  ("glBSP Nodebuilder " GLBSP_VER)

#define WADFNAMESIZE 64

HWND mainwindow;
HINSTANCE maininstance;

// Main window
#define MAIN_WINDOW_X      50
#define MAIN_WINDOW_Y      50
#define MAIN_WINDOW_WIDTH  300
#define MAIN_WINDOW_HEIGHT 140

// File Button
#define MAIN_FILEBUTTON_X       14
#define MAIN_FILEBUTTON_Y       15
#define MAIN_FILEBUTTON_WIDTH   120
#define MAIN_FILEBUTTON_HEIGHT  25
#define MAIN_FILEBUTTON_TEXT    "Choose WAD..."

// File Display Editbox
#define MAIN_FILEEDITBOX_X      137
#define MAIN_FILEEDITBOX_Y      16
#define MAIN_FILEEDITBOX_WIDTH  150
#define MAIN_FILEEDITBOX_HEIGHT 23

// Build Button
#define MAIN_BUILDBUTTON_X      50
#define MAIN_BUILDBUTTON_Y      55
#define MAIN_BUILDBUTTON_WIDTH  200
#define MAIN_BUILDBUTTON_HEIGHT 25
#define MAIN_BUILDBUTTON_TEXT   "Perform Nodebuild"

// Handles
static int filebutton  = -1;  // Choose Wad Handle
static int fileeditbox = -1;  // Filebox handle
static int buildbutton = -1;  // Build Button

// glBSP Settings
static char wadfile[WADFNAMESIZE];
static char waddir[((_MAX_PATH)-2)-(WADFNAMESIZE)];

// ============================ MAIN WINDOW ============================

// Prototypes
boolean_g MainProcCommandMessage(HWND window, HWND ctrl, DWORD command);
boolean_g MainInitBuildButton(HWND wnd, HINSTANCE inst);
boolean_g MainInitFileButton(HWND wnd, HINSTANCE inst);
boolean_g MainInitFileEditbox(HWND wnd, HINSTANCE inst);

//
// MessageHandler
//
// Message handling Loop
//
long MessageHandler(HWND window, UINT message, WPARAM wParam, LPARAM lParam)
{
  switch (message)
  {
    case WM_COMMAND:
      if (MainProcCommandMessage(window, (HWND)(lParam), LOWORD(wParam)))
        return 0L;
      break;

    case WM_DESTROY:
      // Clean up and close the app
      PostQuitMessage(0);
      return 0L;
  }

  return DefWindowProc(window, message, wParam, lParam);
}

//
// MainStartupApplication
//
// Init Main Window
//
boolean_g MainStartupApplication(HINSTANCE inst, INT cmdshow)
{
  WNDCLASS wc;
  char* classname;
  msgfunc_t messagefunc;

  if (!I_GUIRegisterClass(inst, CLTYPE_MAINAPP))
    return false;
  
  classname = I_GUIReturnClassName(CLTYPE_MAINAPP);
  if (!classname)
    return false;

  maininstance = inst;

  // Create a window
  mainwindow = CreateWindowEx(WS_EX_WINDOWEDGE,
                          classname,
                          TITLE,
                          WS_OVERLAPPED|WS_SYSMENU|WS_MINIMIZEBOX|WS_CAPTION,
                          MAIN_WINDOW_X,
                          MAIN_WINDOW_Y,
                          MAIN_WINDOW_WIDTH,
                          MAIN_WINDOW_HEIGHT,
                          NULL,
                          LoadMenu(maininstance, MAKEINTRESOURCE(MAIN_MENU)),
                          maininstance,
                          NULL);

  if (!mainwindow)
    return false;

  // Add Message Handler
  messagefunc.winhandle = mainwindow;
  messagefunc.handler = MessageHandler;
  if (!I_GUIAddMessageHandler(&messagefunc))
  {
    DestroyWindow(mainwindow);
    return false;
  }

  ShowWindow(mainwindow, cmdshow);
  UpdateWindow(mainwindow);
  SetFocus(mainwindow);

  return true;
}

//
// MainProcCommandMessage
//
// Handle WM_COMMAND Messages
//
boolean_g MainProcCommandMessage(HWND window, HWND ctrl, DWORD command)
{
  if (I_GUIButtonMsghandler(ctrl, command))
    return true;

  if (I_GUIEditboxMsghandler(ctrl, command))
    return true;

  switch (command)
  {
    case MAIN_MENU_MAIN_EXIT:
      PostQuitMessage(0);
      return true;

    case MAIN_MENU_HELP_ABOUT:
      MainInitAboutbox();
      return true;

    default:
      break;
  }

  return false;
}

//
// MainInitControls
//
boolean_g MainInitControls(void)
{
  INITCOMMONCONTROLSEX icm;

  // Get Current Directory
  memset (waddir, '\0', _MAX_PATH);
  if (!getcwd(waddir, _MAX_PATH))
    return false;

  // Zero-length the wadfile string
  wadfile[0] = '\0';

  icm.dwSize = sizeof(INITCOMMONCONTROLSEX);
  icm.dwICC = ICC_WIN95_CLASSES;
  InitCommonControlsEx(&icm);

  if (!MainInitBuildButton(mainwindow, maininstance))
    return false;

  if (!MainInitFileButton(mainwindow, maininstance))
    return false;

  if (!MainInitFileEditbox(mainwindow, maininstance))
    return false;

  return true;
}

//
// MainChooseWad
//
void MainChooseWad(void)
{
  filedialog_t fd;
  char *str;
  int slen;
  int i;

  // Setup file dialog
  fd.filters      = "*.WAD";
  fd.filternames  = "WAD Files";
  fd.startdir     = waddir;
  fd.title        = "Choose WAD";
  fd.parentwindow = mainwindow;

  // Open Dialog
  str = I_GUIOpenFileDialog(&fd);
  if (str)
  {
    slen = strlen(str);

    // Too big?
    if (slen>=_MAX_PATH)
      return;

    // Uppercase the string
    i=0;
    while(str[i])
    {
      str[i]=tolower(str[i]);
      i++;
    }

    //
    // Find last directory seperator - we need to seperate filename
    // and path.
    //
    i = (slen-1);
    while (i>0 && str[i] != '\\')
      i--;

    // Check for crap feedback
    if (!i)
      return;

    // Get Directory
    strncpy(waddir, str, i);
    waddir[i] = '\0'; 

    // Get filename
    strncpy(wadfile, &str[i+1], slen-(i+1));   
    wadfile[slen-(i+1)] = '\0'; 

#ifdef DEVELOPERS
    MessageBox(mainwindow, waddir, "NOTICE", MB_OK);
    MessageBox(mainwindow, wadfile, "NOTICE", MB_OK);
#endif /* DEVELOPERS */

    I_GUISetEditboxText(fileeditbox, wadfile);
    free(str);
  }

  return;
}

//
// MainAttemptNodeBuild
//
void MainAttemptNodeBuild(void)
{
  nodebuildinfo_t nb_info;
  nodebuildcomms_t nb_comms;

  char *fullwadname;
  int slen;

  slen = strlen(wadfile);
  if (!slen)
  {
    MessageBox(mainwindow, "Please Choose a WAD File to Node Build", "Notice", MB_OK);
    return;
  }

  slen += strlen(waddir); // WAD Directory is always valid
  slen += 2;              // Include Directory Seperator and NULL

  fullwadname = malloc(sizeof(char)*slen);
  if (!fullwadname)
  {
    MessageBox(mainwindow, "Full WAD Name Malloc Failure!", "Error", MB_OK);
    return;
  }

  // Construct full path and name
  sprintf(fullwadname,"%s%c%s", waddir, '\\', wadfile);

#ifdef DEVELOPERS
  MessageBox(mainwindow, fullwadname, "File wad name", MB_OK);
#endif /* DEVELOPERS */

  nb_info  = default_buildinfo;
  nb_comms = default_buildcomms;
  
  nb_info.input_file = fullwadname;

  // FIXME: check result, and don't build if != GLBSP_E_OK
  GlbspCheckInfo(&nb_info, &nb_comms);

  if (GLBSP_E_OK != GlbspBuildNodes(&nb_info, &gui_build_funcs, &nb_comms))
    MessageBox(mainwindow, "Node Build FAILED", "Notice:", MB_OK);
  else
    MessageBox(mainwindow, "Node Build OK", "Notice:", MB_OK);

  return;
}

// ============================ ABOUT BOX ==============================

//
// MainInitAboutbox
//
boolean_g MainInitAboutbox(void)
{
  return AboutWindowStartup(mainwindow, maininstance);
}

//
// MainKillAboutbox
//
void MainKillAboutbox(void)
{
  AboutWindowShutdown();
  return;
}

// ========================= BUILD PROC BOX ============================

//
// MainInitBuildProc
//
boolean_g MainInitBuildProc(void)
{
  return BuildWindowStartup(mainwindow, maininstance);
}

//
// MainKillBuildProc
//
void MainKillBuildProc(void)
{
  BuildWindowShutdown();
  return;
}

// ========================== FILEPROC BOX =============================

//
// MainInitFileProc
//
boolean_g MainInitFileProc(void)
{
  return FileProcWindowStartup(mainwindow, maininstance);
}

//
// MainKillFileProc
//
void MainKillFileProc(void)
{
  FileProcWindowShutdown();
  return;
}

// ========================= "BUILD" BUTTON ============================

//
// MainBuildButtonHandler
//
void MainBuildButtonHandler(HWND window, DWORD message)
{
  if (message == BN_CLICKED)
    MainAttemptNodeBuild();

  return;
}

//
// MainInitBuildButton
//
boolean_g MainInitBuildButton(HWND wnd, HINSTANCE inst)
{
  guibutton_t button;

  // Already inited?
  if (buildbutton>=0)
    return false;

  memset(&button, 0, sizeof(guibutton_t));

  button.parentwindow     = wnd;
  button.parentinst       = inst;
  button.x                = MAIN_BUILDBUTTON_X;
  button.y                = MAIN_BUILDBUTTON_Y;
  button.width            = MAIN_BUILDBUTTON_WIDTH;
  button.height           = MAIN_BUILDBUTTON_HEIGHT;
  button.text             = MAIN_BUILDBUTTON_TEXT;
  button.action           = MainBuildButtonHandler;

  buildbutton = I_GUICreateButton(&button);

  if (buildbutton < 0)
    return false;

  return true;
}

// ======================= "CHOOSE WAD" BUTTON =========================

//
// MainFileButtonHandler
//
void MainFileButtonHandler(HWND window, DWORD message)
{
  if (message == BN_CLICKED)
    MainChooseWad();

  return;
}

//
// MainInitFileButton
//
boolean_g MainInitFileButton(HWND wnd, HINSTANCE inst)
{
  guibutton_t button;

  // Already inited?
  if (filebutton>=0)
    return false;

  memset(&button, 0, sizeof(guibutton_t));

  button.parentwindow     = wnd;
  button.parentinst       = inst;
  button.x                = MAIN_FILEBUTTON_X;
  button.y                = MAIN_FILEBUTTON_Y;
  button.width            = MAIN_FILEBUTTON_WIDTH;
  button.height           = MAIN_FILEBUTTON_HEIGHT;
  button.text             = MAIN_FILEBUTTON_TEXT;
  button.action           = MainFileButtonHandler;

  filebutton = I_GUICreateButton(&button);

  if (filebutton < 0)
    return false;

  return true;
}

// ========================= WAD File EDITBOX ==========================

//
// MainInitFileEditbox
//
boolean_g MainInitFileEditbox(HWND wnd, HINSTANCE inst)
{
  editbox_t editbox;

  // Already inited?
  if (fileeditbox>=0)
    return false;   

  memset(&editbox, 0, sizeof(editbox_t));

  editbox.parentwindow     = wnd;
  editbox.parentinst       = inst;
  editbox.x                = MAIN_FILEEDITBOX_X;
  editbox.y                = MAIN_FILEEDITBOX_Y;
  editbox.width            = MAIN_FILEEDITBOX_WIDTH;
  editbox.height           = MAIN_FILEEDITBOX_HEIGHT;
  editbox.action           = NULL;                    // No handler

  fileeditbox = I_GUICreateEditbox(&editbox);

  if (fileeditbox < 0)
    return false;

  I_GUISetReadOnlyEditbox(fileeditbox, true);
  return true;
}

// =====================================================================
