//----------------------------------------------------------------------------
//  glBSP Win32 Display Functions
//----------------------------------------------------------------------------
// 
//  Copyright (c) 2000  Andy Baker
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//----------------------------------------------------------------------------
//
#include "sys_inc.h"

// consistency check
#ifndef GLBSP_GUI
#error GLBSP_GUI should be defined when compiling this file
#endif


static displaytype_e currdis = DIS_INVALID;


//
// The Callback Structure
//
const nodebuildfuncs_t gui_build_funcs =
{
  GuiFatalError,
  GuiPrintMsg,
  GuiTicker,

  GuiDisplayOpen,
  GuiDisplaySetTitle,
  GuiDisplaySetText,
  GuiDisplaySetBar,
  GuiDisplaySetBarLimit,
  GuiDisplaySetBarText,
  GuiDisplayClose
};


//
// GuiFatalError
//
void GuiFatalError(const char *str, ...)
{
  // FIXME !!! open window, show message, await click, then ABORT.
  exit(5);
}

//
// GuiPrintMsg
//
void GuiPrintMsg(const char *str, ...)
{
  // FIXME: ideally have scrolling text box (5-10 lines) in MainApp window
}

//
// GuiTicker
//
void GuiTicker(void)
{
  /* does nothing -- perhaps check win32 events ? */
}


//
// GuiDisplayOpen
//
// Setup the display type
//
boolean_g GuiDisplayOpen(displaytype_e type)
{
  boolean_g ret;

  GuiDisplayClose(); // Shutdown any existing display

  // Lets assume failure
  ret = false;

  // Startup new display
  switch(type)
  {
    case DIS_BUILDPROGRESS:
      ret = MainInitBuildProc();
      break;

    case DIS_FILEPROGRESS:
      ret = MainInitFileProc();
      break;

    default:
      break;
  }

  if (ret)
    currdis = type;
  else
    currdis = DIS_INVALID;
  
  return (ret);
}

//
// GuiDisplaySetTitle
//
void GuiDisplaySetTitle(const char *str)
{
  switch(currdis)
  {
    case DIS_BUILDPROGRESS:
      BuildWindowSetTitle(str);
      break;

    case DIS_FILEPROGRESS:
      FileProcWindowSetTitle(str);
      break;

    case DIS_INVALID:
    default:
      break;
  }
}

//
// GuiDisplaySetBar
//
void GuiDisplaySetBar(int barnum, int count)
{
  switch(currdis)
  {
    case DIS_BUILDPROGRESS:
      BuildUpdateBar(barnum, count);
      break;

    case DIS_FILEPROGRESS:
      if (barnum==1)
        FileProcUpdateBar(count);
      break;

    case DIS_INVALID:
    default:
      break;
  }
}

//
// GuiDisplaySetBarLimit
//
void GuiDisplaySetBarLimit(int barnum, int limit)
{
  switch(currdis)
  {
    case DIS_BUILDPROGRESS:
      BuildUpdateBarLimit(barnum, limit);
      break;

    case DIS_FILEPROGRESS:
      if (barnum==1)
        FileProcUpdateBarLimit(limit);
      break;

    case DIS_INVALID:
    default:
      break;
  }
}

//
// GuiDisplaySetBarText
//
// Text above said bar
//
void GuiDisplaySetBarText(int barnum, const char *str)
{
  switch(currdis)
  {
    case DIS_BUILDPROGRESS:
      BuildUpdateBarText(barnum, str);
      break;

    case DIS_FILEPROGRESS:
      if (barnum==1)
        FileProcUpdateBarText(str);
      break;

    case DIS_INVALID:
    default:
      break;
  }
}

//
// GuiDisplaySetText
//
// Text below the bars
//
void GuiDisplaySetText(const char *str)
{
  if (currdis != DIS_BUILDPROGRESS)
    return;

  // *** TODO ***

  // -AJA- Prolly won't be needed after all.
}

//
// GuiDisplayClose
//
// Shutdown active display
//
void GuiDisplayClose(void)
{
  switch(currdis)
  {
    case DIS_BUILDPROGRESS:
      MainKillBuildProc();
      currdis = DIS_INVALID;
      break;

    case DIS_FILEPROGRESS:
      MainKillFileProc();
      currdis = DIS_INVALID;
      break;

    case DIS_INVALID:
    default:
      break;
  }
}

