//----------------------------------------------------------------------------
//  glBSP Win32 System Internal Header
//----------------------------------------------------------------------------
// 
//  Copyright (c) 2000  Andy Baker
// 
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//----------------------------------------------------------------------------

#ifndef __SYSTEM_INTERNAL_H__
#define __SYSTEM_INTERNAL_H__

#include "..\glbsp.h"
#include "glbsp.rh"

#define false  FALSE
#define true   TRUE

#define FLOAT_IEEE_754

#include <windows.h>
#include <commctrl.h>   // includes common control functionality 
#include <commdlg.h>    // includes common dialog functionality 
#include <dlgs.h>       // includes common dialog template defines 
#include <cderr.h>      // includes the common dialog error codes 

#include <ctype.h>
#include <direct.h>
#include <fcntl.h>
#include <io.h>
#include <limits.h>
#include <math.h>
#include <malloc.h>
#include <mem.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys\stat.h>
#include <time.h>

#include <gl/gl.h>


// Window List
typedef enum
{
  WNDL_MAINAPP,     
  WNDL_ABOUTBOX,
  WNDL_BUILDPROC,
  WNDL_FILEPROC,
  NUMOFWINDOWS
}
windowlist_e;

// BOX_ABOUT.C
boolean_g AboutWindowStartup(HWND parentwnd, HINSTANCE inst);
void AboutWindowShutdown(void);

// BOX_BUILD.C
boolean_g BuildWindowStartup(HWND parentwnd, HINSTANCE inst);
boolean_g BuildWindowSetTitle(const char *str);
boolean_g BuildUpdateBarText(int num, const char *str);
boolean_g BuildUpdateBar(int num, int val);
boolean_g BuildUpdateBarLimit(int num, int val);
void BuildWindowShutdown(void);

// BOX_FILEPROC.C
boolean_g FileProcWindowStartup(HWND parentwnd, HINSTANCE inst);
boolean_g FileProcWindowSetTitle(const char *str);
boolean_g FileProcUpdateBarText(const char *str);
boolean_g FileProcUpdateBar(int val);
boolean_g FileProcUpdateBarLimit(int val);
void FileProcWindowShutdown(void);

// BOX_MAIN.C
boolean_g MainStartupApplication(HINSTANCE inst, INT cmdshow);
boolean_g MainInitControls(void);

boolean_g MainInitAboutbox(void); 
boolean_g MainInitBuildProc(void);
boolean_g MainInitFileProc(void);
void MainKillAboutbox(void); 
void MainKillBuildProc(void);
void MainKillFileProc(void);

// DISPLAY.C
extern const nodebuildfuncs_t gui_build_funcs;

void GuiFatalError(const char *str, ...);
void GuiPrintMsg(const char *str, ...);
void GuiTicker(void);
boolean_g GuiDisplayOpen(displaytype_e type);
void GuiDisplaySetTitle(const char *str);
void GuiDisplaySetBar(int barnum, int count);
void GuiDisplaySetBarLimit(int barnum, int limit);
void GuiDisplaySetBarText(int barnum, const char *str);
void GuiDisplaySetText(const char *str);
void GuiDisplayClose(void);

#endif /* __SYSTEM_INTERNAL_H__ */
